////////////////////////////////////////////////////////////////////////////////
// Mechanical trade system for MetaTrader 5                                   //
// Copyright (c) 2005-2011 Interesting                                        //
////////////////////////////////////////////////////////////////////////////////
// Version:      1.0                                                          //
// Version File: 1.0.0.1                                                      //
// File name: DemoEA.mq5                                                      //
////////////////////////////////////////////////////////////////////////////////
//                                 Copyright                                  //
////////////////////////////////////////////////////////////////////////////////
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
////////////////////////////////////////////////////////////////////////////////
//                                  Property                                  //
////////////////////////////////////////////////////////////////////////////////
#property version "1.0"
////////////////////////////////////////////////////////////////////////////////
//           Connection of the additional modules of the trade system         //
////////////////////////////////////////////////////////////////////////////////
#include <Trade\Trade.mqh>
////////////////////////////////////////////////////////////////////////////////
//                              Input parameters                              //
////////////////////////////////////////////////////////////////////////////////
input int TakeProfit = 100; //Take profit (in pips)
input int StopLoss   = 100; //Stop loss (in pips)
input double Lot     = 0.1; //Lot size    
////////////////////////////////////////////////////////////////////////////////
//              Object classes, used in working the trade system              //
////////////////////////////////////////////////////////////////////////////////
//****************************************************************************//
//                              Expert Advisors                               //
//****************************************************************************//
//Class MqlExpert
class CMqlExpert
//Description: The main class of the trading system
{
//----------------------------------------------------------------------------//
protected:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                    Protected declarations of the class                     //
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//----------------------------------------------------------------------------//
//                    Protected variables for object class                    //
//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//
public:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                      Public declarations of the class                      //
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//----------------------------------------------------------------------------//
//                     Public characteristics of the class                    //
//----------------------------------------------------------------------------//
CTrade Trader; //Trading object
//----------------------------------------------------------------------------//
//                         Public methods of the class                        //
//----------------------------------------------------------------------------//
//Main methods
virtual bool Init();
virtual bool Deinit();
//System timer
virtual bool TimerStart(const int interval=1);
virtual bool TimerStop();
//Positions processing (Open long and short positions)
virtual bool MarketBuy(string symbol,double volume,double price,double sl=0,double tp=0,const string comment="");
virtual bool MarketSell(string symbol,double volume,double price,double sl=0,double tp=0,const string comment="");
//----------------------------------------------------------------------------//
//                         Public events of the class                         //
//----------------------------------------------------------------------------//
//Standard events of the trade system
virtual bool OnTick(const string symbol); //Processing the event NewTick (OnTick)
virtual bool OnTimer();                   //Processing the event Timer (OnTimer)
//Specialized events of the trade system
virtual bool OnEvent(const int id, const long& lparam, const double& dparam, const string& sparam);
//----------------------------------------------------------------------------//
};
//             Global variables, used in working the trade system             //
////////////////////////////////////////////////////////////////////////////////
//****************************************************************************//
//                               Global objects                               //
//****************************************************************************//
//Global variables for object classes (Expert Advisors)
CMqlExpert Expert; //Trade expert
//****************************************************************************//
//               Variables for keeping specialized information                //
//****************************************************************************//
//Timer
bool TimerActive = false;
//Hendles for indicators (Spys)
int Hendle_Spy1 = INVALID_HANDLE; //Identifier of the indicator 1
int Hendle_Spy2 = INVALID_HANDLE; //Identifier of the indicator 2
////////////////////////////////////////////////////////////////////////////////
//                                Trade expert                                //
////////////////////////////////////////////////////////////////////////////////
//****************************************************************************//
//                       Expert initialization function                       //
//****************************************************************************//
int OnInit()
{
//----------------------------------------------------------------------------//
//Work variables
int SymbolDigits = 0;

int Result = 0; //Returned importance
//----------------------------------------------------------------------------//

//Forming a list of characters
SymbolSelect("GBPUSD",true);
SymbolSelect("USDJPY",true);
//Initialization of the indicators
Result = Expert.Init();
//Timer
TimerActive = Expert.TimerStart(60);

Print("Processing...");

//----------------------------------------------------------------------------//
return(0);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//                      Expert deinitialization function                      //
//****************************************************************************//
void OnDeinit(const int reason)
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //The result of the processing
//----------------------------------------------------------------------------//

//Indicators
Result = Expert.Deinit();
//Timer
  if(TimerActive)Expert.TimerStop();

//----------------------------------------------------------------------------//      
}
//****************************************************************************//
//                           Expert Ticks function                            //
//****************************************************************************//
void OnTick()
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //The result of the processing
//----------------------------------------------------------------------------//

//Correction of the error initialization
  if(!TimerActive)
  {
    if((Hendle_Spy1==INVALID_HANDLE)||(Hendle_Spy2==INVALID_HANDLE))Expert.Init();
  }
//Processing new tick of the main class of the trading system
Result = Expert.OnTick(Symbol());

//The execution errors checking
  if(!Result)Print("Symbol:",Symbol()," - Error of processing a event OnTick!");
  
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//                           Expert Timer function                            //
//****************************************************************************//
void OnTimer()
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //The result of the processing
//----------------------------------------------------------------------------//

//Correction of the error initialization
  if((Hendle_Spy1==INVALID_HANDLE)||(Hendle_Spy2==INVALID_HANDLE))
  {
  Expert.Init();
  }
//Processing event timer of the main class of the trading system
Result = Expert.OnTimer();

//The execution errors checking
  if(!Result)Print("Timer: Error of processing a event OnTimer!");

//----------------------------------------------------------------------------//   
}
//****************************************************************************//
//                        Expert Chart Event function                         //
//****************************************************************************//
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //The result of the processing
//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//
//                           Processing chart events                          //
//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//
//                           Processing user events                           //
//----------------------------------------------------------------------------//
  if(id>CHARTEVENT_CUSTOM)
  //User event is received
  {
  Result = Expert.OnEvent(id,lparam,dparam,sparam);
  }

//The execution errors checking
  if(!Result)Print("Events: Error of processing a event OnEvent!");  
//----------------------------------------------------------------------------//   
}
////////////////////////////////////////////////////////////////////////////////
//                    Events and methods of object classes                    //
////////////////////////////////////////////////////////////////////////////////
//****************************************************************************//
//                   Class CMqlExpert (Events and methods)                    //
//****************************************************************************//
//Method CMqlExpert.Init
bool CMqlExpert::Init()
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

//Indicators (Main indicators)
  if(Hendle_Spy1==INVALID_HANDLE)Hendle_Spy1 = iCustom("GBPUSD",PERIOD_CURRENT,"SendEvent");
  if(Hendle_Spy2==INVALID_HANDLE)Hendle_Spy2 = iCustom("USDJPY",PERIOD_CURRENT,"SendEvent");

  if((Hendle_Spy1==INVALID_HANDLE)||(Hendle_Spy2==INVALID_HANDLE))
  {
  Print("Error initializing the main indicators!");
  }

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Method CMqlExpert.Deinit
bool CMqlExpert::Deinit()
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

  if(Hendle_Spy1!=INVALID_HANDLE)IndicatorRelease(Hendle_Spy1);
  if(Hendle_Spy2!=INVALID_HANDLE)IndicatorRelease(Hendle_Spy2);

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Method CMqlExpert.TimerStart
bool CMqlExpert::TimerStart(const int interval=1)
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

Result = EventSetTimer(interval);

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Method CMqlExpert.TimerStop
bool CMqlExpert::TimerStop()
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

EventKillTimer();

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Method CMqlExpert.MarketBuy
bool CMqlExpert::MarketBuy(string symbol,double volume,double price,
double sl=0,double tp=0,const string comment="")
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

Result = Trader.PositionOpen(symbol,ORDER_TYPE_BUY,volume,price,sl,tp,comment);

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Method CMqlExpert.MarketSell
bool CMqlExpert::MarketSell(string symbol,double volume,double price,
double sl=0,double tp=0,const string comment="")
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Event CMqlExpert.OnTick
bool CMqlExpert::OnTick(const string symbol)
//Processing the event NewTick (OnTick)
{
//----------------------------------------------------------------------------//
//Work variables
bool ResAsk = 1,ResBid = 1,ResDig = 1,ResPoint;

double NewAsk = 0, NewBid = 0; //Ask and Bid
double Points = 0; //Point
long Dig; //Digits

double PriceOpen = 0,PriceTP = 0,PriceSL =0;

int TP = 0, SL = 0; //TakeProfit and StopLoss

bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

//Information on symbol
ResAsk   = SymbolInfoDouble(symbol,SYMBOL_ASK,NewAsk);   //Ask
ResBid   = SymbolInfoDouble(symbol,SYMBOL_BID,NewBid);   //Bid
ResDig   = SymbolInfoInteger(symbol,SYMBOL_DIGITS,Dig);  //Digits
ResPoint = SymbolInfoDouble(symbol,SYMBOL_POINT,Points); //Point size
//Adjusting the size of the TP and SL
TP = TakeProfit;SL = StopLoss;

  if(Dig==5 || Dig==3){TP = TP*10;SL = SL*10;}

PriceOpen = NormalizeDouble(NewAsk,(int)Dig);
PriceSL   = NormalizeDouble(NewAsk-Points*SL,(int)Dig); 
PriceTP   = NormalizeDouble(NewAsk+Points*TP,(int)Dig);

Result = MarketBuy(symbol,Lot,PriceOpen,PriceSL,PriceTP,"Market Buy");

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Event CMqlExpert.OnTimer
bool CMqlExpert::OnTimer()
//Processing the event Timer (OnTimer)
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
//****************************************************************************//
//Event CMqlExpert.OnEvent
bool CMqlExpert::OnEvent(const int id, const long& lparam, const double& dparam, const string& sparam)
{
//----------------------------------------------------------------------------//
//Work variables
bool Result = true; //Returned importance
//----------------------------------------------------------------------------//

ResetLastError();

  switch(id)
  {
  //Signals from indicator
  case 1001 : Result = Result = OnTick(sparam); break;
  //Default
  default: Result = false;
  }  

//The execution errors checking
  if(GetLastError()!=0)
  {
  Result = false;
  }
//----------------------------------------------------------------------------//
return(Result);
//----------------------------------------------------------------------------//
}
////////////////////////////////////////////////////////////////////////////////
//                               Implementation                               //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////